#define F_CPU 8000000L

#include <inttypes.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "i2cmaster.h"
#include "glcd_routines.h"

#define GLCD_ADDR 0x20

#define BUFFER_DELAY 5


void glcd_clear_screen(void) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_CLEAR_SCREEN);
	i2c_write(0);
	i2c_stop();
	_delay_ms(5);
};

void glcd_set_light(const uint8_t light_level) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_SET_LIGHT);
	i2c_write(light_level);
	i2c_stop();
	_delay_ms(5);
}

void glcd_set_pixel(const uint8_t x, const uint8_t y, const uint8_t mode) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_SET_PIXEL);
	i2c_write(x);
	i2c_write(y);
	i2c_write(mode);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_draw_dec(const uint16_t dec, const uint8_t len, const uint8_t dec_places) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_DEC);
	i2c_write(dec >> 8);
	i2c_write(dec & 0x00ff);
	i2c_write(len);
	i2c_write(dec_places);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_draw_sdec(const int16_t dec, const uint8_t len, const uint8_t dec_places) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_SDEC);
	i2c_write(dec >> 8);
	i2c_write(dec & 0x00ff);
	i2c_write(len);
	i2c_write(dec_places);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_set_cursor(const uint8_t x_pos, const uint8_t y_pos) {
	_delay_us(10);
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_SET_CURSOR);
	i2c_write(x_pos);
	i2c_write(y_pos);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_draw_char(const uint8_t my_char) {
	_delay_us(1);
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_CHAR);
	i2c_write(my_char);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_draw_string(const char *string) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_STRING);
	do {
		i2c_write(*string);
		++string;
	} while (*string > 0);
	i2c_write(0);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

/* glcd_draw_pgm_string ----------------------------------------------------------------------------
* Draws a zero-terminated string that is stored in program memory.
*/
void glcd_draw_pgm_string(const char *my_str)
{
	char my_char;
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_STRING);
	do {
		my_char = pgm_read_byte(my_str);
		i2c_write(my_char);
		++my_str;
	} while (my_char > 0);
	i2c_write(0);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
};

void glcd_draw_rectangle(const uint8_t x_start, const uint8_t y_start, const uint8_t x_end, const uint8_t y_end, const uint8_t mode) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_RECTANGLE);
	i2c_write(x_start);
	i2c_write(y_start);
	i2c_write(x_end);
	i2c_write(y_end);
	i2c_write(mode);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_draw_filled_rectangle(const uint8_t x_start, const uint8_t y_start, const uint8_t x_end, const uint8_t y_end, const uint8_t mode) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_FILLED_RECTANGLE);
	i2c_write(x_start);
	i2c_write(y_start);
	i2c_write(x_end);
	i2c_write(y_end);
	i2c_write(mode);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_draw_center_str(char *string) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_CENTER_STR);
	do {
		i2c_write(*string);
		++string;
	} while (*string > 0);
	i2c_write(0);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_draw_pgm_center_str(const char *my_str) {
	char my_char;
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_CENTER_STR);
	do {
		my_char = pgm_read_byte(my_str);
		i2c_write(my_char);
		++my_str;
	} while (my_char > 0);
	i2c_write(0);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_set_font(const uint8_t font) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_SET_FONT);
	i2c_write(font);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_draw_line(const uint8_t x_start, const uint8_t y_start, const uint8_t x_end, const uint8_t y_end, const uint8_t mode) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_LINE);
	i2c_write(x_start);
	i2c_write(y_start);
	i2c_write(x_end);
	i2c_write(y_end);
	i2c_write(mode);
	i2c_stop();
}

void glcd_draw_dotted_hor_line(const uint8_t y_pos, const uint8_t x_start, const uint8_t x_end, const uint8_t spacing, const uint8_t mode) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_DOTTED_HOR_LINE);
	i2c_write(y_pos);
	i2c_write(x_start);
	i2c_write(x_end);
	i2c_write(spacing);
	i2c_write(mode);
	i2c_stop();
}

void glcd_draw_dotted_ver_line(const uint8_t x_pos, const uint8_t y_start, const uint8_t y_end, const uint8_t spacing, const uint8_t mode) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DRAW_DOTTED_VER_LINE);
	i2c_write(x_pos);
	i2c_write(y_start);
	i2c_write(y_end);
	i2c_write(spacing);
	i2c_write(mode);
	i2c_stop();
}

void glcd_set_verbose(const uint8_t verbose_status) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	if (verbose_status) 
		i2c_write(GLCD_VERBOSE_ON);
	else
		i2c_write(GLCD_VERBOSE_OFF);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_set_autoscroll(const uint8_t scroll_status) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	if (scroll_status)
		i2c_write(GLCD_SET_AUTOSCROLL_ON);
	else
		i2c_write(GLCD_SET_AUTOSCROLL_OFF);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}


void glcd_set_dim(void) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DIM_LIGHT_ON);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
}

void glcd_clear_dim(void) {
	i2c_start_wait(2 * GLCD_ADDR + I2C_WRITE);
	i2c_write(GLCD_DIM_LIGHT_OFF);
	i2c_stop();
	_delay_us(BUFFER_DELAY);
};

