/*************************************************************************************************/
/* keyboard_routines.c */
/*************************************************************************************************/

#ifndef cmh_f_cpu_
#define cmh_f_cpu_

#define F_CPU 8000000L

#endif

#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include "chicken_mobile.h"
#include "glcd_routines.h"
#include "keyboard_routines.h"
#include "display_routines.h"
#include "cmh_util.h"

// local variables
static uint8_t keyboard_buffer[KEYBOARD_BUFFER_SIZE];
static volatile uint8_t keyboard_buffer_wr_pnt = 0;
static uint8_t keyboard_buffer_rd_pnt = 0;

/* init_keyboard ----------------------------------------------------------------------------------
* Sets the keyboard ports 
*/
void init_keyboard(void) {
	KEYBOARD_DDR |= 0b00000011;			// set bit 0 and 1 to output (matrix rows)
	KEYBOARD_DDR &= 0b11100011;			// set bit 2, 3, and 4 to input (matrix columns)
	KEYBOARD_DDR &= 0b11011111;			// set bit 5 to input for external key ("display on")
	KEYBOARD_PORT |= 0b00011111;		// set rows to high, set pull up resisters for keyboard columns
	KEYBOARD_PORT |= 0b00100000;		// set pull-up for external key ("display on")
};
	
/* check_keyboard ---------------------------------------------------------------------------------
* Checks the keyboard for any key pressed
* If the display is off then the display is switched on, but the key is not added to the buffer
* If the key is pressed for at least 3 cycle times, it is added to the keyboard buffer
* Warning: this routine is part of the interrupt service!!
*/
void check_keyboard(void) {
	uint8_t keyboard_value, keyboard_bits;
	static uint8_t keyboard_status = 0;				// manages keyboard decoding status
	static uint8_t keyboard_value_old = 0;			// previous keyboard value, required to check for keyboard bouncing
	extern volatile uint8_t job_flags;
	extern display_status_t display_status;

	// Check whether any key is pressed. Result is stored in "keyboard_value"
	keyboard_value = 0;
	if ((KEYBOARD_INPORT & 0b00100000) == 0) {
		// check external key 
		keyboard_value = 7;
	};
	if (keyboard_value == 0) {
		// Check first keyboard row
		KEYBOARD_PORT &= ~(1 << 0);								// check first keyboard row
		asm volatile("\tnop\n"::);								// wait 1 NOP
		keyboard_bits = (KEYBOARD_INPORT & 0b00011100);			// read keyboard columns
		KEYBOARD_PORT |= (1 << 0);								// reset first keyboard row
		switch (keyboard_bits) {
			case 0b00011000:
				keyboard_value = 1;
				break;
			case 0b00010100:
				keyboard_value = 2;
				break;
			case 0b00001100:
				keyboard_value = 3;
				break;
		};
	};
	if (keyboard_value == 0) {
		// Check second keyboard row
		KEYBOARD_PORT &= ~(1 << 1);								// check first keyboard row
		asm volatile("\tnop\n"::);								// wait 1 NOP
		keyboard_bits = (KEYBOARD_INPORT & 0b00011100);			// read keyboard columns
		KEYBOARD_PORT |= (1 << 1);								// reset first keyboard row
		switch (keyboard_bits) {
			case 0b00011000:
				keyboard_value = 4;
				break;
			case 0b00010100:
				keyboard_value = 5;
				break;
			case 0b00001100:
				keyboard_value = 6;
				break;
		};
	};
	
	// Exit in case that no key was pressed.
	if (keyboard_value == 0) {
			keyboard_status = 0;								// no key pressed -> exit
			return;
	};
	
	// Display on? If not, switch it on and ignore key (wait for key to be released)
	if (display_status.timeout_cnt == 0) {					// display is off - use key to switch on light
		job_flags |= (1 << JF_DISPLAY_LIGHT_ON);					// turn display on
		display_status.timeout_cnt = display_status.timeout;							
		keyboard_status = 0xff;								// wait for key to be released
		return;
	};

	// display is active, key needs to be evaluated
	display_status.timeout_cnt = display_status.timeout;	// reset display timeout
	switch (keyboard_status) {
		case 0:												// key pressed first time
			keyboard_value_old = keyboard_value;			// remember new keyboard value
			++keyboard_status;								// switch keyboard_status to evaluation mode
			break; 
		case 2:												// capture key first time
			if (keyboard_value_old == keyboard_value) {		// still the same key?
				keyboard_buffer[keyboard_buffer_wr_pnt] = keyboard_value;
				if (++keyboard_buffer_wr_pnt == KEYBOARD_BUFFER_SIZE) keyboard_buffer_wr_pnt = 0;
				++keyboard_status;							// keep an eye on how long the key is pressed
			} else keyboard_status = 0;
			break;
		case 15:											// capture repeated key
			if (keyboard_value_old == keyboard_value) {		// still the same key?
				keyboard_buffer[keyboard_buffer_wr_pnt] = keyboard_value;
				if (++keyboard_buffer_wr_pnt == KEYBOARD_BUFFER_SIZE) keyboard_buffer_wr_pnt = 0;
				keyboard_status = 12;
			} else keyboard_status = 0;
			break;		
		case 0xff:											// wait for key to be released	
			break;
		default:
			if (keyboard_value_old == keyboard_value) ++keyboard_status;
			else keyboard_status = 0;						// different key pressed -> reset keyboard routine
			break;
	}
};


/*----------------------------------------------------------------------------------------
* Reads one byte from keyboard buffer.
* Returns: 0 if buffer is empty, otherwise code of pressed key
*/
uint8_t read_keyboard(void) {
	uint8_t keyboard_value;
	
	sleep_mode();											// wait for next interrupt
	run_scheduler();										// take care of scheduled tasks
	if (keyboard_buffer_wr_pnt == keyboard_buffer_rd_pnt) return(0);
	
	keyboard_value = keyboard_buffer[keyboard_buffer_rd_pnt];
	if (++keyboard_buffer_rd_pnt == KEYBOARD_BUFFER_SIZE) keyboard_buffer_rd_pnt = 0;
	return(keyboard_value);
}

/* edit_data_item ------------------------------------------------------------------------*
* Uses the keyboard to edit a data item
* Input: *item - pointer to the 8 bit variable to be edited
*         *str - pointer to the label text (program memory)
*         min - minimal allowed value
*         max - maximal allowed value
* Returns: 0 - okay
*        > 0 - cancel, escape
*/
uint8_t edit_data_item(edit_frame_t *edit_data) {
	uint16_t buffer_value = edit_data->value;
	uint8_t keyboard_value;
	uint8_t stop_flag = 0;
	uint8_t i;
	const char *str = edit_data->string;
	extern display_status_t display_status;
	const uint8_t xpos_data_item = 140;
	
	if (edit_data->value > edit_data->max) edit_data->value = edit_data->max;
	else if (edit_data->value < edit_data->min) edit_data->value = edit_data->min; 
	
	glcd_draw_filled_rectangle(MESSAGE_BOX_X + 1, MESSAGE_BOX_Y + 1, GLCD_MAXX - MESSAGE_BOX_X - 1, GLCD_MAXY - MESSAGE_BOX_Y - 1, 0);
	glcd_draw_rectangle( MESSAGE_BOX_X, MESSAGE_BOX_Y, GLCD_MAXX - MESSAGE_BOX_X, GLCD_MAXY - MESSAGE_BOX_Y, 1);
	show_menu(str);
	// advance string pointer to label text
	for (i = 0; i < 5; ++i) {
		while ((pgm_read_byte(str)) > 0) ++str;
		++str;
	};
	// draw label and data item
	glcd_set_font(0);
	glcd_set_cursor(MESSAGE_BOX_X + 4, MESSAGE_BOX_Y + 8);
	glcd_draw_pgm_string(str);
	glcd_set_cursor(xpos_data_item, MESSAGE_BOX_Y + 8);
	glcd_draw_dec(buffer_value, 5, edit_data->decimal_places);
	
	do {
		if (display_status.timeout_cnt == 0) keyboard_value = 1;	// auto-cancel in case of timeout
		else keyboard_value = read_keyboard();						// wait for key pressed
		switch (keyboard_value) {
			case 1:
				stop_flag = 1;
				break;
			case 2:
				if ((buffer_value > edit_data->step) && (buffer_value - edit_data->step > edit_data->min))
					buffer_value -= edit_data->step;
				else
					buffer_value = edit_data->min;
				glcd_set_cursor(xpos_data_item, MESSAGE_BOX_Y + 8);
				glcd_draw_dec(buffer_value, 5, edit_data->decimal_places);
				break;
			case 3:
				if (buffer_value + edit_data->step < edit_data->max)
					buffer_value += edit_data->step;
				else
					buffer_value = edit_data->max;
				glcd_set_cursor(xpos_data_item, 28);
				glcd_draw_dec(buffer_value, 5, edit_data->decimal_places);
				break;
			case 4:
				stop_flag = 1;
				break;
			case 5:
				if (buffer_value < edit_data->max) {
					++buffer_value;
					glcd_set_cursor(xpos_data_item, 28);
					glcd_draw_dec(buffer_value, 5, edit_data->decimal_places);
				};
				break;
			case 6:
				if (buffer_value > edit_data->min) {
					--buffer_value;
					glcd_set_cursor(xpos_data_item, 28);
					glcd_draw_dec(buffer_value, 5, edit_data->decimal_places);
				};
				break;
			default:
				;
		}
	} while (stop_flag == 0);

	if (keyboard_value == 4) {
		edit_data->value = buffer_value;
		return(0);
	} else {
		return(1);
	};
}


